// Copyright (c) 1997 Andrew Maltsev <am@amsoft.ru>
// All rights reserved
// 
// Dynamic object loader for FreeBSD 2.2
//
// It is known, that it's impossible to statically link executables,
// when this code is in use, because in FreBSD dlopen/dlclose is not
// library's functions, but internal dynamic loader interface. And so
// when there is no dynamic loader compiled in there is no required
// symbols present :(
//
// Appropriate bug report to FreeBSD team had been sent.
//
// $Id: freebsd.C,v 1.4 1998/10/31 17:20:11 am Exp $
//
#include <dlfcn.h>

///////////////////////////////////////////////////////////////////////////////
// Loading (opening) shared object
//
bool
easysql_dynamic_loader::load(char const *file)
{ if(!file) return false;
  unload();
  handle=dlopen((char *)file,1);	// `file' should be const, but it is not!
  if(!handle)
   { set_error("loading error -- %s",dlerror());
     return false;
   }
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Unloading (closing) shared object
//
bool
easysql_dynamic_loader::unload()
{ if(handle)
   { dlclose(handle);
     handle=NULL;
   }
  return true;
}

///////////////////////////////////////////////////////////////////////////////
// Checking object
//
easysql_dynamic_loader::operator bool()
{ return bool(handle);
}

///////////////////////////////////////////////////////////////////////////////
// Returning symbol address from shared object
//
void *
easysql_dynamic_loader::symbol(char const *name)
{ if(!(*this) || !name)
   return NULL;

#ifndef __ELF__
  char buf[strlen(name)+2];
  *buf='_';				// In FreeBSD a.outs symbols have to be
  strcpy(buf+1,name);			// prefixed with `_'
  void *s=dlsym(handle,buf);
#else
  void *s=dlsym(handle,name);
#endif
  if(!s)
   { set_error("can't resolve %s -- %s",name,dlerror());
     return NULL;
   }
  return s;
}
